//
//  KBWebArchiver.m
//  ---------------
//
//  Keith Blount 2005
//

#import "KBWebArchiver.h"
#import "NSURL_ValidityChecking.h"


@interface KBWebArchiver (Private)
- (WebArchive *)archiveFromString:(NSString *)URLString isPath:(BOOL)isPath textString:(NSString **)textString;
@end

@implementation KBWebArchiver

+ (WebArchive *)webArchiveFromURLString:(NSString *)URLString textString:(NSString **)textString
{
	KBWebArchiver *archiver = [[[KBWebArchiver alloc] init] autorelease];
	return [archiver archiveFromString:URLString isPath:NO textString:textString];
}

+ (WebArchive *)webArchiveFromURLPathString:(NSString *)path textString:(NSString **)textString
{
	KBWebArchiver *archiver = [[[KBWebArchiver alloc] init] autorelease];
	return [archiver archiveFromString:path isPath:YES textString:textString];
}

- (WebArchive *)archiveFromString:(NSString *)URLString isPath:(BOOL)isPath textString:(NSString **)textString
{
	WebView *webView = [[WebView alloc] initWithFrame:NSMakeRect(0,0,200,200)];
	
	NSURL *url = (isPath) ? [NSURL fileURLWithPath:URLString] : [NSURL URLWithString:URLString];
	//NSLog (@"isPath: %i, isValid: %i", isPath, [url httpIsValid]);
	if ( (!isPath) && (![url httpIsValid]) )
		return nil;
	
	[webView setFrameLoadDelegate:self];

	finishedLoading = NO;
	loadFailed = NO;
	
	NSURLRequest *theRequest = [NSURLRequest requestWithURL:url];
	
	[[webView mainFrame] loadRequest:theRequest];
	
	// Wait until the site has finished loading (may need to change back to 2 second interval?)
	while (finishedLoading == NO)
		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:1.0]];
	
	[[webView mainFrame] stopLoading];	// Ensure the frame stops loading, otherwise will crash when released!
	[webView setFrameLoadDelegate:nil];
	
	if (loadFailed)
	{
		[webView release];
		return nil;
	}
	
	// Get the text
	if ([[[[webView mainFrame] frameView] documentView] conformsToProtocol:@protocol(WebDocumentText)])
		*textString = [(id <WebDocumentText>)[[[webView mainFrame] frameView] documentView] string];
	else
		*textString = @"";	// Return a blank string

	// the -dataSource method was causing some crashes and also some web pages only half-loaded;
	// using the -DOMDocument method seems to work much better.
	
	[[[[[webView mainFrame] DOMDocument] webArchive] data] writeToFile:@"/users/keithblount/downloads/webTest1.webarchive" atomically:YES];
	
	//return [[[webView mainFrame] dataSource] webArchive];
	return [[[[webView autorelease] mainFrame] DOMDocument] webArchive];
}

// Oh dear, this can cause some crashes - eg. importing Yahoo...

- (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame
{
	if (frame == [sender mainFrame])
		finishedLoading = YES;
}

// Check for errors loading page
- (void)webView:(WebView *)sender didFailProvisionalLoadWithError:(NSError *)error forFrame:(WebFrame *)frame
{
	loadFailed = YES;
	finishedLoading = YES;
}

- (void)webView:(WebView *)sender didFailLoadWithError:(NSError *)error forFrame:(WebFrame *)frame
{
	loadFailed = YES;
	finishedLoading = YES;
}

@end
